from . import consts
import requests
import urllib
import time
import math
from user_agent import generate_user_agent as UserAgent
import re
import base64
# GET the token
def requestToken(sessionID,callback,proxy):
    proxyOptions = None
    nopath = None
    if type(proxy) == type(str()):
        proxy = proxy or ""
    elif proxy and proxy.proxy:
        proxyOptions = proxy.options or {}
        nopath = proxy.nopath
        proxy = proxy.proxy
    else:
        proxy = ""
    uri = None
    if not nopath:
        if proxy and proxy[-1] == "/":
            uri = proxy + "https://" + consts.ENDPOINT_URI + consts.TOKEN_ENDPOINT + str(sessionID) + "/?" + str(math.floor(time.time() * 1000))
        elif proxy:
            uri = proxy + "/https://" + consts.ENDPOINT_URI + consts.TOKEN_ENDPOINT + str(sessionID) + "/?" + str(math.floor(time.time() * 1000))
        else:
            uri = "https://" + consts.ENDPOINT_URI + consts.TOKEN_ENDPOINT + str(sessionID) + "/?" + str(math.floor(time.time() * 1000))
    _uri = urllib.parse.urlparse(uri)
    options = {
        "port": consts.ENDPOINT_PORT,
        "headers": {
            "user-agent": UserAgent(),
            "host": (proxy and uri.hostname) or "kahoot.it",
            "referer": "https://kahoot.it/",
            "accept-language": "en-US,en;q=0.8",
            "accept": "*/*"
        }
    }
    if proxyOptions:
        options.update(proxyOptions)
    r = requests.get(uri);
    if not r.headers.get("x-kahoot-session-token"):
        return callback(None,None,None)
    try:
        data = r.json()
    except Exception as e:
        return callback(None,e,None)
    callback(r.headers.get("x-kahoot-session-token"),data["challenge"],data)
# Evaluate the JS challenge
def solveChallenge(challenge):
    solved = ""
    anti_whitespace = re.compile(r'(\u0009|\u2003)',re.M)
    challenge = anti_whitespace.sub("",challenge)
    challenge = re.sub("this ","this",challenge)
    anti_dot = re.compile(r' *\. *',re.M)
    challenge = anti_dot.sub(".",challenge);
    anti_paren1 = re.compile(r' *\( *',re.M)
    anti_paren2 = re.compile(r' *\) *',re.M)
    challenge = anti_paren1.sub("(",challenge)
    challenge = anti_paren2.sub(")",challenge)
    challenge = re.sub("console.","",challenge);
    challenge = re.sub("this\.angular\.isObject\(offset\)", "true",challenge);
    challenge = re.sub("this\.angular\.isString\(offset\)", "true",challenge);
    challenge = re.sub("this\.angular\.isDate\(offset\)", "true",challenge);
    challenge = re.sub("this\.angular\.isArray\(offset\)", "true",challenge);
    challenge = _challengeToPython(challenge)
    edict={}
    exec(challenge,globals(),edict)
    return edict["fin"]
# Convert the JS to Python
def _challengeToPython(challenge):
    challenge = re.sub(r"\.call\(this,","(",challenge)
    challenge = re.sub(r";\s*","\n",challenge)
    challenge = re.compile('function',re.M).sub("def",challenge)
    challenge = re.compile(r'\)\{',re.M).sub("):\n\t",challenge)
    challenge = re.sub("true","True",challenge)
    challenge = re.compile(r"char\.charCodeAt\(0\)").sub("ord(char)",challenge)
    challenge = re.compile(r"String\.fromCharCode").sub("chr",challenge)
    challenge = re.sub(r"var\s+offset","offset",challenge)
    challenge = re.compile(r"if\(True\)").sub("\tif(True):\n\t\t",challenge)
    challenge = re.compile("return",re.M).sub("\treturn",challenge)
    challenge = re.compile(r"\s+def\(char",re.M).sub("\n\tdef repl(char",challenge)
    challenge = re.compile(r"return\s+_",re.M).sub("#",challenge)
    challenge = re.compile(r"message\):",re.M).sub("message):\n\tdef log(a,b,c):\n\t\tpass\n",challenge)
    # challenge = "def log(a,b,c):\n\tpass\n" + challenge
    call = re.search(r"decode\(.*\)$",challenge,re.M).group(0)
    challenge = re.compile(r"decode\(.*\)$",re.M).sub("",challenge)
    challenge = re.compile(r"^\}\)?",re.M).sub("",challenge)
    challenge = re.compile(r"\)\n\n",re.M).sub(")\n\tres=\"\"\n\tfor i in range(0,len(message)):\n\t\tres+=repl(message[i],i)\n\treturn res",challenge)
    challenge += "\nfin=" + call
    return challenge
# Decode the token header
def decodeBase64(base):
    try:
        base += "=" * ((4 - len(base) % 4) % 4)
        return base64.b64decode(base).decode("utf-8")
    except Exception as e:
        return e
# complex stuff to get the actual token
def concatTokens(headerToken,challengeToken):
    token = ""
    for i in range(0,len(headerToken)):
        char = ord(headerToken[i])
        mod = ord(challengeToken[i % len(challengeToken)])
        decodedChar = char ^ mod
        token += chr(decodedChar)
    return token;
# main function
def resolve(sessionID,callback,proxy=None):
    if str(sessionID)[0] == "0":
        return requestChallenge(sessionID,callback,proxy)
    def _(headerToken,challenge,gamemode):
        if not headerToken:
            return callback(None,None)
        token1 = decodeBase64(headerToken)
        token2 = solveChallenge(challenge)
        resolvedToken = concatTokens(token1,token2)
        callback(resolvedToken,gamemode)
    requestToken(sessionID,_,proxy)
# getting challenge stuff
def requestChallenge(sessionID,callback,proxy):
    proxyOptions = None
    nopath = None
    if type(proxy) == type(str()):
        proxy = proxy or ""
    elif proxy and proxy.proxy:
        proxyOptions = proxy.options or {}
        nopath = proxy.nopath
        proxy = proxy.proxy
    else:
        proxy = ""
    uri = None
    if not nopath:
        if proxy[-1] == "/":
            uri = proxy + "https://" + consts.ENDPOINT_URI + consts.CHALLENGE_ENDPOINT + "/pin/" + sessionID
        else:
            uri = proxy + "/https://" + consts.ENDPOINT_URI + consts.CHALLENGE_ENDPOINT + "/pin/" + sessionID
    _uri = urllib.parse.urlparse(uri)
    options = {
        "port": consts.ENDPOINT_PORT,
        "headers": {
            "user-agent": UserAgent(),
            "host": (proxy and uri.hostname) or "kahoot.it",
            "referer": "https://kahoot.it/",
            "accept-language": "en-US,en;q=0.8",
            "accept": "*/*"
        }
    }
    if proxyOptions:
        options.update(proxyOptions)
    r = requests.get(uri);
    try:
        data = r.json()
    except Exception as e:
        return callback(None,e,None)
    try:
        inf = {
            "twoFactorAuth": False,
            "gameMode": data.challenge.type,
            "kahootData": data.kahoot,
            "rawChallengeData": data.challenge
        }.update(data.challenge.game_options)
        return callback(True,inf)
    except Exception as e:
        return callback(None,e,None)



import asyncio
import websocket
from . import consts
from pymitter import EventEmitter
from user_agent import generate_user_agent as UserAgent
import time
import math
from random import random as random
import json as JSON
try:
    import thread
except ImportError:
    import _thread as thread

class WSHandler(EventEmitter):
    def __init__(self,session,token,kahoot):
        super().__init__()
        self.requires2Step = False
        self.finished2Step = False
        self.kahoot = kahoot
        self.msgID = 0
        self.receivedQuestionTime = 0
        self.clientID = "_none_"
        self.connected = False
        self.gameID = session
        self.name = ""
        self.ready = False
        self.firstQuizEvent = False
        self.timesync = None
        self.lastReceivedQ = None
        def on_message(ws,m):
            self.message(m)
        def on_close(ws):
            #print("close")
            self.connected = False
            self.close()
        def on_error(ws,e):
            #print(e)
            self.connected = False
            self.emit("error",e)
            self.close()
        def on_open(ws):
            self.open(ws)
        self.ws = websocket.WebSocketApp(consts.WSS_ENDPOINT + str(session) + "/" + token,on_message=on_message,on_error=on_error,on_close=on_close)
        self.ws.on_open = on_open
        def _1(data,content):
            if not getattr(self.kahoot,"quiz"):
                self.emit("quizData",{
                    "name": None,
                    "type": content["quizType"],
                    "qCount": content["quizQuestionAnswers"][0],
                    "totalQ": len(content["quizQuestionAnswers"]),
                    "quizQuestionAnswers": content["quizQuestionAnswers"]
                })
            if not getattr(self.kahoot.quiz,"currentQuestion"):
                self.emit("quizUpdate", {
                    "questionIndex": content["questionIndex"],
                    "timeLeft": content["timeLeft"],
                    "type": content["gameBlockType"],
                    "useStoryBlocks": content.get("canAccessStoryBlocks"),
                    "ansMap": content.get("answerMap")
                })
            elif content["questionIndex"] > self.kahoot.quiz.currentQuestion.index:
                self.emit("quizUpdate", {
                    "questionIndex": content["questionIndex"],
                    "timeLeft": content["timeLeft"],
                    "type": content.get("gameBlockType"),
                    "useStoryBlocks": content.get("canAccessStoryBlocks"),
                    "ansMap": content.get("answerMap")
                })

        def _2(data,content):
            self.receivedQuestionTime = math.floor(time.time() * 1000)
            self.emit("questionStart")

        def _3(data,content):
            self.emit("finish", {
                "playerCount": content["playerCount"],
                "quizID": content["quizId"],
                "rank": content["rank"],
                "correct": content["correctCount"],
                "incorrect": content["incorrectCount"]
            })

        def _8(data,content):
            self.emit("questionEnd", {
                "correctAnswers": content["correctAnswers"],
                "correct": content["isCorrect"],
                "points": content["points"],
                "pointsData": content["pointsData"],
                "rank": content["rank"],
                "nemesis": content.get("nemesis"),
                "text": content.get("text"),
                "totalScore": content["totalScore"]
            })

        def _9(data,content):
            if not self.firstQuizEvent:
                self.firstQuizEvent = True
                self.emit("quizData", {
                    "name": content["quizName"],
                    "type": content.get("quizType"),
                    "qCount": content["quizQuestionAnswers"][0],
                    "totalQ": len(content["quizQuestionAnswers"]),
                    "quizQuestionAnswers": content["quizQuestionAnswers"]
                })

        def _10(data,content):
            self.emit("quizEnd")
            try:
                self.ws.close()
            except Exception as e:
                # probably already closed
                pass

        def _12(data,content):
            self.emit("feedback")

        def _13(data,content):
            self.emit("finishText", {
                "metal": content["podiumMedalType"]
            })

        def _51(data,content):
            self.emit("2StepFail")

        def _52(data,content):
            if not self.finished2Step:
                self.finished2Step = True
                self.emit("2StepSuccess")

        def _53(data,content):
            self.requires2Step = True
            if not self.finished2Step:
                self.emit("2Step")

        self.dataHandler = {
            "1": _1,
            "2": _2,
            "3": _3,
            "8": _8,
            "9": _9,
            "10": _10,
            "12": _12,
            "13": _13,
            "51": _51,
            "52": _52,
            "53": _53
        }
    def getExt(self):
        return {
            ack: True,
            timesync: {
                l: 0,
                o: 0,
                tc: math.floor(time.time() * 1000)
            }
        }
    def getPacket(self, packet):
        l = None
        o = None
        try:
            l = round(((time.time() * 1000) - packet["ext"]["timesync"]["tc"] - packet["ext"]["timesync"]["p"]) / 2)
            o = packet["ext"]["timesync"]["ts"] - packet["ext"]["timesync"]["tc"] - l;
            self.timesync = {
                "tc": round(time.time() * 1000),
                "l": l,
                "o": o
            }
        except Exception as e:
            self.timesync = {
                "tc": round(time.time() * 1000),
                "l": 0,
                "o": 0
            }
        self.msgID+=1
        return [{
            "channel": packet["channel"],
            "clientId": self.clientID,
            "ext": {
                "ack": packet["ext"]["ack"],
                "timesync": {
                    "l": l,
                    "o": o,
                    "tc": round(time.time() * 1000)
                }
            },
            "id": str(self.msgID)
        }]
    def getSubmitPacket(self, questionChoice, question):
        self.msgID+=1
        r = [{
            "channel": "/service/controller",
            "clientId": self.clientID,
            "data": {
                "content": JSON.dumps({
                    "choice": questionChoice,
                    "questionIndex": self.kahoot.quiz.currentQuestion.index,
                    "meta": {
                        "lag": round(random() * 45 + 5)
                    },
                    "type": question.type
                }),
                "gameid": self.gameID,
                "host": consts.ENDPOINT_URI,
                "id": 45,
                "type": "message"
            },
            "id": str(self.msgID)
        }]
        if question.type == "open_ended" or question.type == "word_cloud":
            r[0].data.content = JSON.dumps({
                text: str(questionChoice),
                questionIndex: self.kahoot.quiz.currentQuestion.index,
                meta: {
                    lag: round(random() * 45 + 5)
                },
                type: question.type
            })
        elif question.type == "multiple_select_quiz" or question.type == "jumble":
            if type(questionChoice.append) != type(chr):
                r[0].data.content = JSON.dumps({
                    choice: [0,1,2,3],
                    questionIndex: self.kahoot.quiz.currentQuestion.index,
                    meta: {
                        lag: round(random() * 45 + 5)
                    },
                    type: question.type
                })
        return r
    def send(self, msg):
        if self.kahoot.loggingMode:
            print("SND: " + JSON.dumps(msg))
        if self.connected:
            try:
                self._ws.send(JSON.dumps(msg))
            except Exception as e:
                print(e)
                pass
    def sendSubmit(self, questionChoice, question, secret=None):
        timer = round(time.time() * 1000) - self.receivedQuestionTime
        if timer < 250:
            time.sleep(0.25 - (timer/1000))
        packet = self.getSubmitPacket(questionChoice,question)
        self.send(packet)
        self.emit("questionSubmit")
    def open(self,ws):
        self._ws = ws
        self.connected = True
        self.emit("open")
        r = [{
            "advice": {
                "interval": 0,
                "timeout": 60000
            },
            "channel": consts.CHANNEL_HANDSHAKE,
            "ext": {
                "ack": True,
                "timesync": {
                    "l": 0,
                    "o": 0,
                    "tc": round(time.time() * 1000)
                },
                "id": "1",
                "minimumVersion": "1.0",
                "supportedConnectionTypes": [
                    "websocket",
                    "long-polling"
                ],
                "version": "1.0"
            }
        }]
        self.msgID+=1
        self.send(r)
        pass
    def message(self, msg):
        if self.kahoot.loggingMode:
            print("DWN: " + msg)
        data = JSON.loads(msg)[0]
        if data.get("channel") == consts.CHANNEL_HANDSHAKE and data.get("error"):
            self.emit("error")
            self.close()
            return
        if data.get("channel") == consts.CHANNEL_HANDSHAKE and data.get("clientId"):
            self.clientID = data.get("clientId")
            r = self.getPacket(data)[0]
            r["advice"] = {
                "timeout": 0
            }
            r["channel"] = "/meta/connect"
            r["connectionType"] = "websocket"
            r["ext"]["ack"] = 0
            self.send([r])
        elif data.get("channel") == consts.CHANNEL_CONN and data.get("advice") and data.get("advice").get("reconnect") and data.get("advice").get("reconnect") == "retry":
            connectionPacket = {
                "ext": {
                    "ack": 1,
                    "timesync": self.timesync
                },
                "id": str(self.msgID + 1)
            }
            self.msgID += 1
            connectionPacket["channel"] = consts.CHANNEL_CONN
            connectionPacket["clientId"] = self.clientID
            connectionPacket["connectionType"] = "websocket"
            self.send([connectionPacket])
            time.sleep(0.5)
            self.ready = True
            self.emit("ready")
        elif data.get("data"):
            if data.get("data").get("error"):
                if data.get("data").get("type") and data.get("data").get("type") == "loginResponse":
                    return self.emit("invalidName")
                try:
                    self.emit("error",data["data"]["error"])
                except Exception as e:
                    pass
                return
            elif data.get("data").get("type") == "loginResponse":
                self.kahoot.cid = data["data"]["cid"]
                self.emit("joined")
            else:
                if data.get("data").get("content"):
                    cont = JSON.loads(data.get("data").get("content"))
                    if self.dataHandler.get(str(data.get("data").get("id"))):
                        self.dataHandler.get(str(data["data"]["id"]))(data,cont)
        if data.get("ext") and data["channel"] == consts.CHANNEL_CONN and not data.get("advice") and self.ready:
            packet = self.getPacket(data)[0]
            packet["connectionType"] = "websocket"
            self.send([packet])
    def send2Step(self, steps):
        packet = [{
            "channel": "/service/controller",
            "clientId": self.clientID,
            "data": {
                "id": 50,
                "type": "message",
                "gameid": self.gameID,
                "host": consts.ENDPOINT_URI,
                "content": JSON.dumps({
                    "sequence": steps
                })
            },
            "id": str(self.msgID)
        }]
        self.msgID+=1
        self.send(packet)
    def sendFeedback(self, fun, learning, recommend, overall):
        packet = [{
            "channel": "/service/controller",
            "clientId": self.clientID,
            "data": {
                "id": 11,
                "type": "message",
                "gameid": self.gameID,
                "host": consts.ENDPOINT_URI,
                "content": JSON.stringify({
                    "totalScore": self.kahoot.totalScore,
                    "fun": fun,
                    "learning": learning,
                    "recommend": recommend,
                    "overall": overall,
                    "nickname": self.kahoot.name
                })
            },
            "id": str(self.msgID)
        }]
        self.msgID += 1
        time.sleep(1)
        self.send(packet)
    def relog(self, cid):
        if not self.ready:
            time.sleep(0.5)
            self.relog(cid)
            return
        self.msgID += 1
        packet = {
            "channel": "/service/controller",
            "clientId": self.clientID,
            "data": {
                "cid": cid,
                "content": JSON.dumps({
                    "device": {
                        "userAgent": UserAgent(),
                        "screen": {
                            "width": 2000,
                            "height": 1000
                        }
                    }
                }),
                "gameid": self.gameID,
                "host": "kahoot.it",
                "type": "relogin"
            },
            "ext": {},
            "id": str(self.msgID)
        }
        self.send([packet])
    def login(self, name, team):
        if not self.ready:
            time.sleep(0.5)
            self.login(name,team)
            return
        self.name = name
        self.msgID+=1
        joinPacket = [{
            "channel": "/service/controller",
            "clientId": self.clientID,
            "data": {
                "content": '{"device":{"userAgent":"' + UserAgent() + '","screen":{"width":1280,"height":800}}}',
                "gameid": self.gameID,
                "host": consts.ENDPOINT_URI,
                "name": self.name,
                "type": "login"
            },
            "ext": {},
            "participantUserId": None,
            "id": str(self.msgID)
        }]
        time.sleep(0.5)
        self.send(joinPacket)
        if self.kahoot.gamemode == "team":
            joinPacket2 = [{
                "channel": "/service/controller",
                "clientId": self.clientID,
                "data": {
                    "content": JSON.dumps(team if type(team) == type(list()) else team if len(team) else ["Player 1", "Player 2", "Player 3", "Player 4"]),
                    "gameid": self.gameID,
                    "host": consts.ENDPOINT_URI,
                    "id": 18,
                    "type": "message"
                },
                "ext": {},
                "participantUserId": None,
                "id": str(self.msgID)
            }]
            self.msgID+=1
            time.sleep(0.5)
            self.send(joinPacket2)
    def close(self):
        self.connected = False
        self.ws.close()
        self.emit("close")
    def leave(self):
        self.msgID+=1
        try:
            self.timesync["tc"] = round(time.time() * 1000)
        except:
            pass
        self.send([{
            "channel": "/meta/disconnect",
            "clientId": self.clientID,
            "ext": {
                "timesync": self.timesync
            },
            "id": str(self.msgID)
        }])
        time.sleep(0.5)
        self.close()



from pymitter import EventEmitter
from .src import token
from .src import Assets
from .src.WSHandler import WSHandler
# from .src.ChallengeHandler import ChallengeHandler
from .src import consts
import threading
import time

class client(EventEmitter):
    def __init__(self,proxies={},options={}):
        super().__init__()
        self._wsHandler = None
        self.token = None
        self.sessionID = None
        self.name = None
        self.team = []
        self.quiz = None
        self.totalScore = 0
        self.gamemode = None
        self.hasTwoFactorAuth = False
        self.usesNamerator = False
        self.cid = ""
        self.proxies = proxies
        self.loggingMode = False
        self.joined = False
        self.options = {
            "ChallengeAutoContinue": True,
            "ChallengeGetFullScore": False
        }.update(options)
    def reconnect(self):
        if self.sessionID and this.cid and this._wsHandler and this._wsHandler.ws.open:
            if self.sessionID[0] == "0":
                return False
            def _(resolvedToken,gamemode):
                self.gamemode = content.get("gameMode") or "classic"
                self.hasTwoFactorAuth = content.get("twoFactorAuth") or False
                self.usesNamerator = content.get("namerator") or False
                self.token = resolvedToken
                self._wsHandler = WSHandler(self.sessionID,self.token,self)
                _defineListeners(self,self._wsHandler)
                thread.start()
            try:
                content = token.resolve(self.sessionID,self.proxies)
            except Exception:
                return False
            return True
    def join(self,pin,name,team=["Player 1","Player 2","Player 3","Player 4"]):
        if not pin or not name:
            return False
        self.sessionID = pin
        self.name = name
        self.team = team
        def _(resolvedToken,content):
            self.gamemode = content.get("gameMode") or "classic"
            self.hasTwoFactorAuth = content.get("twoFactorAuth") or False
            self.usesNamerator = content.get("namerator") or False
            self.token = resolvedToken
            self._wsHandler = WSHandler(self.sessionID,self.token,self)
            _defineListeners(self,self._wsHandler)
            thread = threading.Thread(target=self._wsHandler.ws.run_forever)
            thread.start()
        try:
            content = token.resolve(self.sessionID,_,self.proxies)
        except Exception:
            print(ConnectError)
            return False
    def answer2Step(self,steps):
        self._wsHandler.send2Step(steps)
    def answerQuestion(self,id,question,secret={}):
        if not question:
            question = self.quiz.currentQuestion
        self._wsHandler.sendSubmit(id,question,secret)
    def leave(self):
        self._wsHandler.leave()
    def sendFeedback(self,fun=1,learning=1,recommend=1,overall=5):
        self._wsHandler.sendFeedback(fun,learning,recommend,overall)
    def next(self):
        if self.gamemode == "challenge":
            self._wsHandler.next()
            return True
        return False
def _defineListeners(client,socket):
    def errorHandle(e):
        client.emit("handshakeFailed",e)
    def invalidNameHandle():
        client.emit("invalidName")
    def TwoFailHandle():
        client.emit("2StepFail")
    def TwoSuccessHandle():
        client.emit("2StepSuccess")
    def TwoHandle():
        client.emit("2Step")
    def ReadyHandle():
        socket.login(client.name,client.team)
    def JoinHandle():
        client.emit("ready")
        client.emit("joined")
        if client.hasTwoFactorAuth:
            client.emit("2Step")
    def QuizDataHandle(quizInfo):
        client.quiz = Assets.Quiz(quizInfo.get("name"),quizInfo["type"],quizInfo.get("qCount"),client,quizInfo["totalQ"],quizInfo["quizQuestionAnswers"],quizInfo)
        client.emit("quizStart",client.quiz)
        client.emit("quiz",client.quiz)
    def QuizUpdateHandle(updateInfo):
        client.quiz.currentQuestion = Assets.Question(updateInfo,client)
        client.emit("question",client.quiz.currentQuestion)
    def QuestionEndHandle(endInfo):
        e = Assets.QuestionEndEvent(endInfo,client)
        client.totalScore = e.total
        client.emit("questionEnd",e)
    def QuizEndHandle():
        client.emit("quizEnd")
        client.emit("disconnect")
    def QuestionStartHandle():
        try:
            client.emit("questionStart",client.quiz.currentQuestion)
        except Exception as e:
            # likely joined during quiz
            pass
    def QuestionSubmitHandle(message=None):
        e = Assets.QuestionSubmitEvent(message,client)
        client.emit("questionSubmit",e)
    def FinishTextHandle(data):
        e = Assets.FinishTextEvent(data)
        client.emit("finishText",e)
    def FinishHandle(data):
        e = Assets.QuizFinishEvent(data,client)
        client.emit("finish",e)
    def FeedbackHandle():
        client.emit("feedback")
    socket.on("error",errorHandle)
    socket.on("invalidName",invalidNameHandle)
    socket.on("2StepFail",TwoFailHandle)
    socket.on("2StepSuccess",TwoSuccessHandle)
    socket.on("2Step",TwoHandle)
    socket.on("ready",ReadyHandle)
    socket.on("joined",JoinHandle)
    socket.on("quizData",QuizDataHandle)
    socket.on("quizUpdate",QuizUpdateHandle)
    socket.on("questionEnd",QuestionEndHandle)
    socket.on("quizEnd",QuizEndHandle)
    socket.on("questionStart",QuestionStartHandle)
    socket.on("questionSubmit",QuestionSubmitHandle)
    socket.on("finishText",FinishTextHandle)
    socket.on("finish",FinishHandle)
    socket.on("feedback",FeedbackHandle)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="KahootPY",
    version="0.1.0",
    author="theusaf",
    author_email="theusafyt@gmail.com",
    description="A python package to interact with Kahoot!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/theusaf/kahoot.js-updated/tree/python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    keywords=["kahoot","bot"],
    install_requires=["websocket-client","pymitter","requests","user_agent"],
)



ENDPOINT_URI = "kahoot.it"
ENDPOINT_PORT = 443
TOKEN_ENDPOINT = "/reserve/session/"
CHALLENGE_ENDPOINT = "/rest/challenges/"
WSS_ENDPOINT = "wss://kahoot.it/cometd/"
CHANNEL_HANDSHAKE = "/meta/handshake"
CHANNEL_SUBSCR = "/meta/subscribe"
CHANNEL_UNSUBSCR = "/meta/unsubscribe"
CHANNEL_CONN = "/meta/connect"
SUPPORTED_CONNTYPES = (
	"websocket",
	"long-polling"
)
EMOJI_REGEX = "/\uD83C\uDFF4\uDB40\uDC67\uDB40\uDC62(?:\uDB40\uDC77\uDB40\uDC6C\uDB40\uDC73|\uDB40\uDC73\uDB40\uDC63\uDB40\uDC74|\uDB40\uDC65\uDB40\uDC6E\uDB40\uDC67)\uDB40\uDC7F|\uD83D\uDC69\u200D\uD83D\uDC69\u200D(?:\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67]))|\uD83D\uDC68(?:\uD83C\uDFFF\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB-\uDFFE])|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFE\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB-\uDFFD\uDFFF])|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFD\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB\uDFFC\uDFFE\uDFFF])|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFC\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB\uDFFD-\uDFFF])|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFB\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFC-\uDFFF])|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\u200D(?:\u2764\uFE0F\u200D(?:\uD83D\uDC8B\u200D)?\uD83D\uDC68|(?:\uD83D[\uDC68\uDC69])\u200D(?:\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67]))|\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67])|(?:\uD83D[\uDC68\uDC69])\u200D(?:\uD83D[\uDC66\uDC67])|[\u2695\u2696\u2708]\uFE0F|\uD83D[\uDC66\uDC67]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|(?:\uD83C\uDFFF\u200D[\u2695\u2696\u2708]|\uD83C\uDFFE\u200D[\u2695\u2696\u2708]|\uD83C\uDFFD\u200D[\u2695\u2696\u2708]|\uD83C\uDFFC\u200D[\u2695\u2696\u2708]|\uD83C\uDFFB\u200D[\u2695\u2696\u2708])\uFE0F|\uD83C[\uDFFB-\uDFFF])|\uD83E\uDDD1(?:(?:\uD83C[\uDFFB-\uDFFF])\u200D(?:\uD83E\uDD1D\u200D\uD83E\uDDD1(?:\uD83C[\uDFFB-\uDFFF])|\uD83C[\uDF3E\uDF73\uDF7C\uDF84\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\u200D(?:\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83C[\uDF3E\uDF73\uDF7C\uDF84\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD]))|\uD83D\uDC69(?:\u200D(?:\u2764\uFE0F\u200D(?:\uD83D\uDC8B\u200D(?:\uD83D[\uDC68\uDC69])|\uD83D[\uDC68\uDC69])|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFF\u200D(?:\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFE\u200D(?:\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFD\u200D(?:\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFC\u200D(?:\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFB\u200D(?:\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD]))|\uD83D\uDC69\uD83C\uDFFF\u200D\uD83E\uDD1D\u200D(?:\uD83D[\uDC68\uDC69])(?:\uD83C[\uDFFB-\uDFFE])|\uD83D\uDC69\uD83C\uDFFE\u200D\uD83E\uDD1D\u200D(?:\uD83D[\uDC68\uDC69])(?:\uD83C[\uDFFB-\uDFFD\uDFFF])|\uD83D\uDC69\uD83C\uDFFD\u200D\uD83E\uDD1D\u200D(?:\uD83D[\uDC68\uDC69])(?:\uD83C[\uDFFB\uDFFC\uDFFE\uDFFF])|\uD83D\uDC69\uD83C\uDFFC\u200D\uD83E\uDD1D\u200D(?:\uD83D[\uDC68\uDC69])(?:\uD83C[\uDFFB\uDFFD-\uDFFF])|\uD83D\uDC69\uD83C\uDFFB\u200D\uD83E\uDD1D\u200D(?:\uD83D[\uDC68\uDC69])(?:\uD83C[\uDFFC-\uDFFF])|\uD83D\uDC69\u200D\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC69\u200D\uD83D\uDC69\u200D(?:\uD83D[\uDC66\uDC67])|(?:\uD83D\uDC41\uFE0F\u200D\uD83D\uDDE8|\uD83D\uDC69(?:\uD83C\uDFFF\u200D[\u2695\u2696\u2708]|\uD83C\uDFFE\u200D[\u2695\u2696\u2708]|\uD83C\uDFFD\u200D[\u2695\u2696\u2708]|\uD83C\uDFFC\u200D[\u2695\u2696\u2708]|\uD83C\uDFFB\u200D[\u2695\u2696\u2708]|\u200D[\u2695\u2696\u2708])|\uD83C\uDFF3\uFE0F\u200D\u26A7|\uD83E\uDDD1(?:(?:\uD83C[\uDFFB-\uDFFF])\u200D[\u2695\u2696\u2708]|\u200D[\u2695\u2696\u2708])|\uD83D\uDC3B\u200D\u2744|(?:\uD83C[\uDFC3\uDFC4\uDFCA]|\uD83D[\uDC6E\uDC70\uDC71\uDC73\uDC77\uDC81\uDC82\uDC86\uDC87\uDE45-\uDE47\uDE4B\uDE4D\uDE4E\uDEA3\uDEB4-\uDEB6]|\uD83E[\uDD26\uDD35\uDD37-\uDD39\uDD3D\uDD3E\uDDB8\uDDB9\uDDCD-\uDDCF\uDDD6-\uDDDD])(?:\uD83C[\uDFFB-\uDFFF])\u200D[\u2640\u2642]|(?:\u26F9|\uD83C[\uDFCB\uDFCC]|\uD83D\uDD75)(?:\uFE0F\u200D[\u2640\u2642]|(?:\uD83C[\uDFFB-\uDFFF])\u200D[\u2640\u2642])|\uD83C\uDFF4\u200D\u2620|(?:\uD83C[\uDFC3\uDFC4\uDFCA]|\uD83D[\uDC6E-\uDC71\uDC73\uDC77\uDC81\uDC82\uDC86\uDC87\uDE45-\uDE47\uDE4B\uDE4D\uDE4E\uDEA3\uDEB4-\uDEB6]|\uD83E[\uDD26\uDD35\uDD37-\uDD39\uDD3C-\uDD3E\uDDB8\uDDB9\uDDCD-\uDDCF\uDDD6-\uDDDF])\u200D[\u2640\u2642])\uFE0F|\uD83D\uDC69\u200D\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67])|\uD83C\uDFF3\uFE0F\u200D\uD83C\uDF08|\uD83D\uDC69\u200D\uD83D\uDC67|\uD83D\uDC69\u200D\uD83D\uDC66|\uD83D\uDC15\u200D\uD83E\uDDBA|\uD83C\uDDFD\uD83C\uDDF0|\uD83C\uDDF6\uD83C\uDDE6|\uD83C\uDDF4\uD83C\uDDF2|\uD83D\uDC08\u200D\u2B1B|\uD83E\uDDD1(?:\uD83C[\uDFFB-\uDFFF])|\uD83D\uDC69(?:\uD83C[\uDFFB-\uDFFF])|\uD83C\uDDFF(?:\uD83C[\uDDE6\uDDF2\uDDFC])|\uD83C\uDDFE(?:\uD83C[\uDDEA\uDDF9])|\uD83C\uDDFC(?:\uD83C[\uDDEB\uDDF8])|\uD83C\uDDFB(?:\uD83C[\uDDE6\uDDE8\uDDEA\uDDEC\uDDEE\uDDF3\uDDFA])|\uD83C\uDDFA(?:\uD83C[\uDDE6\uDDEC\uDDF2\uDDF3\uDDF8\uDDFE\uDDFF])|\uD83C\uDDF9(?:\uD83C[\uDDE6\uDDE8\uDDE9\uDDEB-\uDDED\uDDEF-\uDDF4\uDDF7\uDDF9\uDDFB\uDDFC\uDDFF])|\uD83C\uDDF8(?:\uD83C[\uDDE6-\uDDEA\uDDEC-\uDDF4\uDDF7-\uDDF9\uDDFB\uDDFD-\uDDFF])|\uD83C\uDDF7(?:\uD83C[\uDDEA\uDDF4\uDDF8\uDDFA\uDDFC])|\uD83C\uDDF5(?:\uD83C[\uDDE6\uDDEA-\uDDED\uDDF0-\uDDF3\uDDF7-\uDDF9\uDDFC\uDDFE])|\uD83C\uDDF3(?:\uD83C[\uDDE6\uDDE8\uDDEA-\uDDEC\uDDEE\uDDF1\uDDF4\uDDF5\uDDF7\uDDFA\uDDFF])|\uD83C\uDDF2(?:\uD83C[\uDDE6\uDDE8-\uDDED\uDDF0-\uDDFF])|\uD83C\uDDF1(?:\uD83C[\uDDE6-\uDDE8\uDDEE\uDDF0\uDDF7-\uDDFB\uDDFE])|\uD83C\uDDF0(?:\uD83C[\uDDEA\uDDEC-\uDDEE\uDDF2\uDDF3\uDDF5\uDDF7\uDDFC\uDDFE\uDDFF])|\uD83C\uDDEF(?:\uD83C[\uDDEA\uDDF2\uDDF4\uDDF5])|\uD83C\uDDEE(?:\uD83C[\uDDE8-\uDDEA\uDDF1-\uDDF4\uDDF6-\uDDF9])|\uD83C\uDDED(?:\uD83C[\uDDF0\uDDF2\uDDF3\uDDF7\uDDF9\uDDFA])|\uD83C\uDDEC(?:\uD83C[\uDDE6\uDDE7\uDDE9-\uDDEE\uDDF1-\uDDF3\uDDF5-\uDDFA\uDDFC\uDDFE])|\uD83C\uDDEB(?:\uD83C[\uDDEE-\uDDF0\uDDF2\uDDF4\uDDF7])|\uD83C\uDDEA(?:\uD83C[\uDDE6\uDDE8\uDDEA\uDDEC\uDDED\uDDF7-\uDDFA])|\uD83C\uDDE9(?:\uD83C[\uDDEA\uDDEC\uDDEF\uDDF0\uDDF2\uDDF4\uDDFF])|\uD83C\uDDE8(?:\uD83C[\uDDE6\uDDE8\uDDE9\uDDEB-\uDDEE\uDDF0-\uDDF5\uDDF7\uDDFA-\uDDFF])|\uD83C\uDDE7(?:\uD83C[\uDDE6\uDDE7\uDDE9-\uDDEF\uDDF1-\uDDF4\uDDF6-\uDDF9\uDDFB\uDDFC\uDDFE\uDDFF])|\uD83C\uDDE6(?:\uD83C[\uDDE8-\uDDEC\uDDEE\uDDF1\uDDF2\uDDF4\uDDF6-\uDDFA\uDDFC\uDDFD\uDDFF])|[#*0-9]\uFE0F\u20E3|(?:\uD83C[\uDFC3\uDFC4\uDFCA]|\uD83D[\uDC6E\uDC70\uDC71\uDC73\uDC77\uDC81\uDC82\uDC86\uDC87\uDE45-\uDE47\uDE4B\uDE4D\uDE4E\uDEA3\uDEB4-\uDEB6]|\uD83E[\uDD26\uDD35\uDD37-\uDD39\uDD3D\uDD3E\uDDB8\uDDB9\uDDCD-\uDDCF\uDDD6-\uDDDD])(?:\uD83C[\uDFFB-\uDFFF])|(?:\u26F9|\uD83C[\uDFCB\uDFCC]|\uD83D\uDD75)(?:\uD83C[\uDFFB-\uDFFF])|(?:[\u261D\u270A-\u270D]|\uD83C[\uDF85\uDFC2\uDFC7]|\uD83D[\uDC42\uDC43\uDC46-\uDC50\uDC66\uDC67\uDC6B-\uDC6D\uDC72\uDC74-\uDC76\uDC78\uDC7C\uDC83\uDC85\uDCAA\uDD74\uDD7A\uDD90\uDD95\uDD96\uDE4C\uDE4F\uDEC0\uDECC]|\uD83E[\uDD0C\uDD0F\uDD18-\uDD1C\uDD1E\uDD1F\uDD30-\uDD34\uDD36\uDD77\uDDB5\uDDB6\uDDBB\uDDD2-\uDDD5])(?:\uD83C[\uDFFB-\uDFFF])|(?:[\u231A\u231B\u23E9-\u23EC\u23F0\u23F3\u25FD\u25FE\u2614\u2615\u2648-\u2653\u267F\u2693\u26A1\u26AA\u26AB\u26BD\u26BE\u26C4\u26C5\u26CE\u26D4\u26EA\u26F2\u26F3\u26F5\u26FA\u26FD\u2705\u270A\u270B\u2728\u274C\u274E\u2753-\u2755\u2757\u2795-\u2797\u27B0\u27BF\u2B1B\u2B1C\u2B50\u2B55]|\uD83C[\uDC04\uDCCF\uDD8E\uDD91-\uDD9A\uDDE6-\uDDFF\uDE01\uDE1A\uDE2F\uDE32-\uDE36\uDE38-\uDE3A\uDE50\uDE51\uDF00-\uDF20\uDF2D-\uDF35\uDF37-\uDF7C\uDF7E-\uDF93\uDFA0-\uDFCA\uDFCF-\uDFD3\uDFE0-\uDFF0\uDFF4\uDFF8-\uDFFF]|\uD83D[\uDC00-\uDC3E\uDC40\uDC42-\uDCFC\uDCFF-\uDD3D\uDD4B-\uDD4E\uDD50-\uDD67\uDD7A\uDD95\uDD96\uDDA4\uDDFB-\uDE4F\uDE80-\uDEC5\uDECC\uDED0-\uDED2\uDED5-\uDED7\uDEEB\uDEEC\uDEF4-\uDEFC\uDFE0-\uDFEB]|\uD83E[\uDD0C-\uDD3A\uDD3C-\uDD45\uDD47-\uDD78\uDD7A-\uDDCB\uDDCD-\uDDFF\uDE70-\uDE74\uDE78-\uDE7A\uDE80-\uDE86\uDE90-\uDEA8\uDEB0-\uDEB6\uDEC0-\uDEC2\uDED0-\uDED6])|(?:[#*0-9\xA9\xAE\u203C\u2049\u2122\u2139\u2194-\u2199\u21A9\u21AA\u231A\u231B\u2328\u23CF\u23E9-\u23F3\u23F8-\u23FA\u24C2\u25AA\u25AB\u25B6\u25C0\u25FB-\u25FE\u2600-\u2604\u260E\u2611\u2614\u2615\u2618\u261D\u2620\u2622\u2623\u2626\u262A\u262E\u262F\u2638-\u263A\u2640\u2642\u2648-\u2653\u265F\u2660\u2663\u2665\u2666\u2668\u267B\u267E\u267F\u2692-\u2697\u2699\u269B\u269C\u26A0\u26A1\u26A7\u26AA\u26AB\u26B0\u26B1\u26BD\u26BE\u26C4\u26C5\u26C8\u26CE\u26CF\u26D1\u26D3\u26D4\u26E9\u26EA\u26F0-\u26F5\u26F7-\u26FA\u26FD\u2702\u2705\u2708-\u270D\u270F\u2712\u2714\u2716\u271D\u2721\u2728\u2733\u2734\u2744\u2747\u274C\u274E\u2753-\u2755\u2757\u2763\u2764\u2795-\u2797\u27A1\u27B0\u27BF\u2934\u2935\u2B05-\u2B07\u2B1B\u2B1C\u2B50\u2B55\u3030\u303D\u3297\u3299]|\uD83C[\uDC04\uDCCF\uDD70\uDD71\uDD7E\uDD7F\uDD8E\uDD91-\uDD9A\uDDE6-\uDDFF\uDE01\uDE02\uDE1A\uDE2F\uDE32-\uDE3A\uDE50\uDE51\uDF00-\uDF21\uDF24-\uDF93\uDF96\uDF97\uDF99-\uDF9B\uDF9E-\uDFF0\uDFF3-\uDFF5\uDFF7-\uDFFF]|\uD83D[\uDC00-\uDCFD\uDCFF-\uDD3D\uDD49-\uDD4E\uDD50-\uDD67\uDD6F\uDD70\uDD73-\uDD7A\uDD87\uDD8A-\uDD8D\uDD90\uDD95\uDD96\uDDA4\uDDA5\uDDA8\uDDB1\uDDB2\uDDBC\uDDC2-\uDDC4\uDDD1-\uDDD3\uDDDC-\uDDDE\uDDE1\uDDE3\uDDE8\uDDEF\uDDF3\uDDFA-\uDE4F\uDE80-\uDEC5\uDECB-\uDED2\uDED5-\uDED7\uDEE0-\uDEE5\uDEE9\uDEEB\uDEEC\uDEF0\uDEF3-\uDEFC\uDFE0-\uDFEB]|\uD83E[\uDD0C-\uDD3A\uDD3C-\uDD45\uDD47-\uDD78\uDD7A-\uDDCB\uDDCD-\uDDFF\uDE70-\uDE74\uDE78-\uDE7A\uDE80-\uDE86\uDE90-\uDEA8\uDEB0-\uDEB6\uDEC0-\uDEC2\uDED0-\uDED6])\uFE0F|(?:[\u261D\u26F9\u270A-\u270D]|\uD83C[\uDF85\uDFC2-\uDFC4\uDFC7\uDFCA-\uDFCC]|\uD83D[\uDC42\uDC43\uDC46-\uDC50\uDC66-\uDC78\uDC7C\uDC81-\uDC83\uDC85-\uDC87\uDC8F\uDC91\uDCAA\uDD74\uDD75\uDD7A\uDD90\uDD95\uDD96\uDE45-\uDE47\uDE4B-\uDE4F\uDEA3\uDEB4-\uDEB6\uDEC0\uDECC]|\uD83E[\uDD0C\uDD0F\uDD18-\uDD1F\uDD26\uDD30-\uDD39\uDD3C-\uDD3E\uDD77\uDDB5\uDDB6\uDDB8\uDDB9\uDDBB\uDDCD-\uDDCF\uDDD1-\uDDDD])/g"



class Quiz():
    def __init__(self, name, type, answerCount, client, amount, answers, rawData):
        self.client = client
        self.name = name
        self.type = type
        self.answerCount = answerCount
        self.currentQuestion = None
        self.questions = []
        self.questionCount = amount
        self.answerCounts = answers
        self.rawEvent = rawData

class Question():
    def __init__(self,rawEvent,client):
        self.client = client
        self.quiz = client.quiz
        self.index = rawEvent["questionIndex"]
        self.timeLeft = rawEvent["timeLeft"]
        self.type = rawEvent["type"]
        self.usesStoryBlocks = rawEvent.get("useStoryBlocks")
        self.ended = False
        self.quiz.questions.append(self)
        self.number = len(self.quiz.questions)
        self.quiz.currentQuestion = self
        self.rawEvent = rawEvent
    def answer(self,number,secret={}):
        if not number and number != 0:
            return False
        self.client.answerQuestion(number,self,secret)

class QuestionEndEvent():
    def __init__(self,rawEvent,client):
        try:
            self.client = client
            self.quiz = client.quiz
            self.question = self.quiz.questions[-1]
            self.question.ended = True
            self.correctAnswers = rawEvent.get("correctAnswers")
            self.correctAnswer = self.correctAnswers[0]
            self.text = rawEvent.get("text")
            self.correct = rawEvent["correct"]
            self.nemesis = Nemesis(rawEvent.get("nemesis"))
            self.points = rawEvent["points"]
            self.rank = rawEvent["rank"]
            self.total = rawEvent["totalScore"]
            self.streak = rawEvent["pointsData"]["answerStreakPoints"]["streakLevel"]
            self.rawEvent = rawEvent
        except Exception as e:
            return e

class QuestionSubmitEvent():
    def __init__(self,message,client):
        self.client = client
        self.quiz = client.quiz
        self.question = self.quiz.questions[-1]
        self.rawEvent = message

class Nemesis():
    def __init__(self,rawData):
        if rawData:
            self.name = rawData.get("name")
            self.score = rawData.get("totalScore")
            self.isGhost = rawData.get("isGhost")
            self.exists = True
            self.rawEvent = rawData
        else:
            self.name = None
            self.score = None
            self.isKicked = None
            self.exists = False
            self.rawEvent = None

class FinishTextEvent():
    def __init__(self,rawEvent):
        self.metal = rawEvent["metal"]
        self.rawEvent = rawEvent

class QuizFinishEvent():
    def __init__(self,rawEvent,client):
        self.client = client
        self.quiz = client.quiz
        self.players = rawEvent["playerCount"]
        self.quizID = rawEvent["quizID"]
        self.rank = rawEvent["rank"]
        self.correct = rawEvent["correct"]
        self.incorrect = rawEvent["incorrect"]
        self.rawEvent = rawEvent
